#!/bin/bash

set -euo pipefail

function echoerr {
    >&2 echo "$@"
}

_usage="Usage: $0 [--(open-ports|close-ports) <RESTAPIPort>  <RPCPort>]
where:
    --open-ports: open RESTAPI and RPC ports on host required for Antrea-NSX.
    --close-ports: close RESTAPI and RPC ports on host which were opened for Antrea-NSX."

function print_usage {
    echoerr "$_usage"
}

OPEN=false
CLOSE=false
IPTABLES="iptables-nft"
INTERFACE_NAME=""
RESTAPI_PORT=10081
RPC_PORT=10082

while [[ $# -gt 0 ]]
 do
 key="$1"

  case $key in
    --open-ports)
      OPEN=true
      RESTAPI_PORT=$2
      RPC_PORT=$3
      shift 3
      ;;
    --close-ports)
      CLOSE=true
      RESTAPI_PORT=$2
      RPC_PORT=$3
      shift 3
      ;;
    -h|--help)
      print_usage
      exit 0
      ;;
    -*)    # unknown option
      echoerr "Unknown option $1"
      exit 1
      ;;
 esac
 done

if [ ${OPEN} == true ] && [ ${CLOSE} == true ]; then
  echoerr "Arguments --open-ports and --close-ports cannot be used together."
  exit 1
fi

function check_host_iptables_mode {
    policy=$(iptables-legacy -L INPUT -v -n -t filter | head -n1 | awk '{print $4}' || true)
    if [[ ${policy} == "DROP" ]]; then
        IPTABLES="iptables-legacy"
    fi
}

function get_interface_name {
    INTERFACE_NAME=$(ip addr show | grep ${NSX_Management_Proxy_POD_IP} | awk '{print $7}')
}

function check_if_rule_exist {
    echo "$(${IPTABLES} -L INPUT -n -v -t filter | grep "ACCEPT.*${INTERFACE_NAME}.*tcp dpt:${1}")"
}

check_host_iptables_mode

get_interface_name

ports=($RESTAPI_PORT $RPC_PORT)

if [[ ${OPEN} == true ]]; then
    for port in "${ports[@]}";
    do
        rule=$(check_if_rule_exist ${port})
        if [[ ${rule} == "" ]]; then
            ${IPTABLES} -I INPUT --in-interface ${INTERFACE_NAME} -p tcp --dport ${port} -j ACCEPT -m comment --comment "opened for nsx-management-proxy"
        fi
    done
elif [[ ${CLOSE} == true ]]; then
    for port in "${ports[@]}";
    do
        rule=$(check_if_rule_exist ${port})
        if [[ ${rule} != "" ]]; then
            ${IPTABLES} -D INPUT --in-interface ${INTERFACE_NAME} -p tcp --dport ${port} -j ACCEPT -m comment --comment "opened for nsx-management-proxy"
        fi
    done
fi
